@@ -1,25 +1,22 @@
 from . import BVH as PybindBVH
 from . import PointsBVH as PybindPointsBVH
-from QtQmlViewport import Product
+from QtQmlViewport import Product, utils
 from QtQmlViewport.Array import Array
 
 from OpenGL import GL as gl
-from PyQt5.QtCore import QObject, Q_ENUMS
-
+from PyQt5.QtCore import QObject, Q_ENUMS, pyqtSlot as Slot
+from PyQt5.QtGui import QVector3D
 import numpy as np
 
 class Attribs( Product.Product ):
     def __init__( self, parent=None, vertices = None, normals = None ):
         super(Attribs, self).__init__( parent )
-        self._vertices = None
-        self._normals = None
-
         self.vertices = vertices
         self.normals = normals
 
 
-    Product.InputProperty(vars(), Array, 'vertices')
-    Product.InputProperty(vars(), Array, 'normals')
+    Product.InputProperty(vars(), Array, 'vertices', None)
+    Product.InputProperty(vars(), Array, 'normals', None)
 
     def get_attributes(self):
         '''
@@ -44,9 +41,6 @@ class BVH( Product.Product ):
 
     def __init__( self, parent=None, indices=None, points=None, primitive_type = PrimitiveType.TRIANGLES):
         super(BVH, self).__init__( parent )
-        self._indices = None
-        self._points = None
-        self._primitiveType = PrimitiveType.TRIANGLES
 
         self.indices = indices
         self.points = points
@@ -58,11 +52,11 @@ class BVH( Product.Product ):
 
     Q_ENUMS(PrimitiveType)
 
-    Product.InputProperty(vars(), int, 'primitiveType')
+    Product.InputProperty(vars(), int, 'primitiveType', PrimitiveType.TRIANGLES)
 
-    Product.InputProperty(vars(), Array, 'indices')
+    Product.InputProperty(vars(), Array, 'indices', None)
 
-    Product.InputProperty(vars(), Array, 'points')
+    Product.InputProperty(vars(), Array, 'points', None)
 
     def _update(self):
         if self._indices is None or self._points is None:
@@ -92,9 +86,6 @@ class Geometry( Product.Product ):
 
     def __init__( self, parent=None, indices = None, attribs = None, primitive_type = PrimitiveType.TRIANGLES ):
         super(Geometry, self).__init__( parent )
-        self._indices = None
-        self._attribs = None
-        self._primitiveType = PrimitiveType.TRIANGLES
         self.bvh = None
 
         self.indices = indices
@@ -105,11 +96,30 @@ class Geometry( Product.Product ):
 
     Q_ENUMS(PrimitiveType)
 
-    Product.InputProperty(vars(), int, 'primitiveType')
+    Product.InputProperty(vars(), int, 'primitiveType', PrimitiveType.TRIANGLES)
 
-    Product.InputProperty(vars(), Array, 'indices')
+    Product.InputProperty(vars(), Array, 'indices', None)
 
-    Product.InputProperty(vars(), Attribs, 'attribs')
+    Product.InputProperty(vars(), Attribs, 'attribs', None)
+
+    @Slot(int, QVector3D, str, result = QVector3D)
+    def faceAttribtAt(self, id, tuv, attribute):
+        face = self.faceIndices(id)
+        va, vb, vc = getattr(self.attribs, attribute).ndarray[face]
+        p = tuv[1] * va + tuv[2] * (vb + 1 - tuv[1] - tuv[2]) * vc
+        return utils.from_numpy(p)
+
+    @Slot(int, result = list)
+    def faceIndices(self, id):
+        assert self.primitiveType == PrimitiveType.TRIANGLES
+        return self.indices.ndarray[id*3:id*3+3].tolist()
+
+    @Slot(int, result = QVector3D)
+    def faceNormal(self, id):
+        face = self.faceIndices(id)
+        va, vb, vc = self.attribs.vertices.ndarray[face]
+        p = np.cross(vb-va, vc-va)
+        return utils.from_numpy(p/np.linalg.norm(p))
 
     def goc_bvh(self, update = False):
 
